/*
  File: specifique.h

  (C) 2011 Philbob

  This program is free software: you can redistribute it and/or modify it
  under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 2.1 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
//-----------------------------------------------------------------------
//	specifique.h
//-----------------------------------------------------------------------


#define classe_e extern
#include<commun.h>
#undef classe_e


//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
#ifndef specifique_h
#define specifique_h
//-----------------------------------------------------------------------
//classe triode
class triode
{
private:

	
	FILE *f;			//pointeur de fichier
	FILE *f_dat;		//pointeur de fichier des courbes numérisées
	FILE *f_cfg;		//pointeur de fichier de config du tube
	FILE *f_prm;		//pointeur de fichier des parametres 
	FILE *f_cmd;		//pointeur de fichier des commandes gnuplot 
	FILE *f_chg;		//pointeur de fichier de la droite de charge 
	FILE *f_mut;		//pointeur de fichier de la courbe mutuelle 
	FILE *f_tmp;		//pointeur de fichier enregistrement temporaire des parametres 
	FILE *f_inc;		//pointeur de fichier modele LTSpice 


	char chemin[len];			//chemin du repertoire de travail
	char nom[len];				//nom du tube
	char tube_dat[len];			//nom du fichier des courbes numérisées
	char tube_cfg[len];			//nom du fichier de config du tube
	char tube_prm[len];			//nom du fichier des parametres du tube
	char tube_chg[len];			//nom du fichier de la droite de charge
	char tube_mut[len];			//nom du fichier de la courbe mutuelle
	char tube_tmp[len];			//nom du fichier des enregistrements temporaires
	char tube_inc[len];			//nom du fichier du modele LTSpice
	
	long np;				//nombre de parametres du modele
	double *pm;					//parametres de départ
	double *p;					//parametres pour le solveur

	double Ia0;			//courant à tension nulle (droite de charge pour les coeffs de poids)
	double Va0;			//tension à courant nul (droite de charge pour les coeffs de poids)
	double A;			//coeffs de la droite de charge Ia=A.Va+B
	double B;
	double resist;		//resistance de charge
	
	long nc;			//nombre de courbes numérisées
	long *npc;			//nombre de points par courbe
	double *Vgn;		//tension de grille des courbes
	double **Van;		//tension de plaque des courbes
	double **Ian;		//courant de plaque des courbes
	double **poids;		//poids des couples Va,Ia par rapport à la droite de charge
	double Van_min;		//tension plaque min
	double Van_max;		//tension plaque max
	double Ian_min;		//courant plaque min
	double Ian_max;		//courant plaque max
	double coefpoids;	//coefficient de pondération de la droite de charge
	double errdeb;		//erreur avant optimisation
	double errfin;		//erreur après optimisation
	long niter;			//nombre d'itérations
	long niterp;		//nombre d'itérations partielles
	double Pw_max;		//puissance max de plaque
	double gain_max;	//gain max compte tenu de la droite de charge
	double polar;		//polar idéale (pour le gain max)

	double *Vgc;		//tension de grille explorant la droite de charge
	double *Iac;		//courant de plaque explorant la droite de charge
	double *Vac;		//tension de plaque explorant la droite de charge
	double *dVac;		//dérivée de la tension de plaque explorant la droite de charge
	long ndc;			//nombre de points de la droite de charge
	
	long opt;			//optimisation 0=non 1=oui
	
	double ccg;			//capacité parasite grille cathode
	double cgp;			//capacité parasite grille plaque
	double ccp;			//capacité parasite plaque cathode
	
	long niag;			//nombre de points du réseau ia=f(vg)
	double *vaiag;		//tension du réseau ia=f(vg)
	double *vgiag;		//tension du réseau ia=f(vg)
	double *iaiag;		//courant du réseau ia=f(vg)

	double Rgi;			//courant de grille

	//routine de lecture du fichier de config du tube
	void lecture_cfg(void);
	
	//chargement des courbes numérisées
	void lecture_dat(void);
	
	//dichotomie
	double dicho(double vg,double va1,double va2);
	
	
public:

	//constructeur
	triode(void);
	
	//destructeur
	~triode(void);
	
	//init
	void init(void);

	//opti
	bool opti(void);

	//calcul de Ip le courant de plaque
	double Ip(double Ep,double Eg);
	
	//calcul de l'erreur
	double erreur(double Va,double Vg,double Ia); 

	//calcul de l'erreur
	double erreurexpo(double Va,double Vg); 
	
	//calcul des erreurs
	double erreur(void);
	
	//optimise
	bool optimise(double _prop);
	
	//sauve param
	void sauve_param(void);
	
	//courbe mutuelle
	void mutuelle(void);
	
	//tracé courbes
	void trace(void);
	
	//fichier LTSpice
	void LTSpice(void);
};
//-----------------------------------------------------------------------
//mini
double mini(double v1,double v2);
//-----------------------------------------------------------------------
//maxi
double maxi(double v1,double v2);
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
//-----------------------------------------------------------------------
#endif
